@@ -1,14 +1,14 @@
 #!/usr/bin/env python
 
-from validictory.validator import SchemaValidator, ValidationError, SchemaError
+from validictory.validator import SchemaValidator, FieldValidationError, ValidationError, SchemaError
 
-__all__ = ['validate', 'SchemaValidator', 'ValidationError', 'SchemaError']
-__version__ = '0.8.2'
+__all__ = ['validate', 'SchemaValidator', 'FieldValidationError', 'ValidationError', 'SchemaError']
+__version__ = '0.9.0'
 
 
 def validate(data, schema, validator_cls=SchemaValidator,
              format_validators=None, required_by_default=True,
-             blank_by_default=False):
+             blank_by_default=False, disallow_unknown_properties=False):
     '''
     Validates a parsed json document against the provided schema. If an
     error is found a :class:`ValidationError` is raised.
@@ -23,8 +23,11 @@ def validate(data, schema, validator_cls=SchemaValidator,
     :param format_validators: optional dictionary of custom format validators
     :param required_by_default: defaults to True, set to False to make
         ``required`` schema attribute False by default.
+    :param disallow_unknown_properties: defaults to False, set to True to
+        disallow properties not listed in the schema definition
     '''
-    v = validator_cls(format_validators, required_by_default, blank_by_default)
+    v = validator_cls(format_validators, required_by_default, blank_by_default,
+                      disallow_unknown_properties)
     return v.validate(data, schema)
 
 if __name__ == '__main__':
